@@ -220,6 +220,14 @@ impl<T, A: FixedSizeArray<T>> FixedCapacityDequeLike<T, A> {
             None
         }
     }
+
+    fn as_slice(&self) -> &[T] {
+        unsafe { &(*self.array.as_ptr()).as_slice()[self.begining..self.end] }
+    }
+
+    fn as_mut_slice(&mut self) -> &mut [T] {
+        unsafe { &mut (*self.array.as_mut_ptr()).as_mut_slice()[self.begining..self.end] }
+    }
 }
 
 impl<T, A: FixedSizeArray<T>> Drop for FixedCapacityDequeLike<T, A> {
@@ -230,6 +238,56 @@ impl<T, A: FixedSizeArray<T>> Drop for FixedCapacityDequeLike<T, A> {
     }
 }
 
+impl<T, A: FixedSizeArray<T>> Debug for FixedCapacityDequeLike<T, A>
+where
+    T: Debug,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("FixedCapacityDequeLike")
+            .field("array", &self.as_slice())
+            .field("begining", &self.begining)
+            .field("end", &self.end)
+            .field("_element", &self._element)
+            .finish()
+    }
+}
+
+impl<T, A: FixedSizeArray<T>> PartialEq for FixedCapacityDequeLike<T, A>
+where
+    T: PartialEq,
+{
+    fn eq(&self, other: &Self) -> bool {
+        self.capacity() == other.capacity()
+            && self.begining == other.begining
+            && self.end == other.end
+            && self.as_slice() == other.as_slice()
+    }
+}
+
+impl<T, A: FixedSizeArray<T>> Eq for FixedCapacityDequeLike<T, A> where T: Eq {}
+
+impl<T, A: FixedSizeArray<T>> Clone for FixedCapacityDequeLike<T, A>
+where
+    T: Clone,
+{
+    fn clone(&self) -> Self {
+        let mut clone = FixedCapacityDequeLike {
+            array: MaybeUninit::uninit(),
+            begining: self.begining,
+            end: self.end,
+            _element: PhantomData,
+        };
+
+        unsafe {
+            for (src, dst) in self.as_slice().iter().zip(clone.as_mut_slice()) {
+                ptr::write(dst, src.clone());
+            }
+        }
+
+        clone
+    }
+}
+
 /// Attempts to initialize array with iterator.
 ///
 /// # Examples
@@ -436,6 +494,39 @@ pub struct ArrayIntoIterator<T, A: FixedSizeArray<T>> {
     deque: FixedCapacityDequeLike<T, A>,
 }
 
+impl<T, A: FixedSizeArray<T>> PartialEq for ArrayIntoIterator<T, A>
+where
+    T: PartialEq,
+{
+    fn eq(&self, other: &Self) -> bool {
+        self.deque.as_slice() == other.deque.as_slice()
+    }
+}
+
+impl<T, A: FixedSizeArray<T>> Eq for ArrayIntoIterator<T, A> where T: Eq {}
+
+impl<T, A: FixedSizeArray<T>> Clone for ArrayIntoIterator<T, A>
+where
+    T: Clone,
+{
+    fn clone(&self) -> Self {
+        ArrayIntoIterator {
+            deque: self.deque.clone(),
+        }
+    }
+}
+
+impl<T, A: FixedSizeArray<T>> Debug for ArrayIntoIterator<T, A>
+where
+    T: Debug,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("ArrayIntoIterator")
+            .field("deque", &self.deque)
+            .finish()
+    }
+}
+
 impl<T, A: FixedSizeArray<T>> ArrayIntoIterator<T, A> {
     pub fn new(array: A) -> ArrayIntoIterator<T, A> {
         ArrayIntoIterator {
@@ -532,6 +623,26 @@ impl<T, CHUNK: FixedSizeArray<T>, STUMP: FixedSizeArray<T>> Eq for ArrayChunk<T,
 {
 }
 
+impl<T, CHUNK: FixedSizeArray<T>, STUMP: FixedSizeArray<T>> Clone for ArrayChunk<T, CHUNK, STUMP>
+where
+    T: Clone,
+{
+    fn clone(&self) -> Self {
+        match self {
+            ArrayChunk::Chunk(chunk, _) => {
+                let chunk_clone: CHUNK =
+                    try_init_from_iterator(chunk.as_slice().iter().cloned()).unwrap();
+                ArrayChunk::Chunk(chunk_clone, PhantomData)
+            }
+            ArrayChunk::Stump(stump, _) => {
+                let stump_clone: STUMP =
+                    try_init_from_iterator(stump.as_slice().iter().cloned()).unwrap();
+                ArrayChunk::Stump(stump_clone, PhantomData)
+            }
+        }
+    }
+}
+
 /// An iterator yielding chunks ("subarrays") of requested size, akin to `core::slice::Chunks`.
 ///
 /// If array can't be evenly divided into chunks, the last item will be a so called stump - an
@@ -587,6 +698,53 @@ pub struct ArrayChunks<T, A: FixedSizeArray<T>, CHUNK: FixedSizeArray<T>, STUMP:
     _stump_pd: PhantomData<STUMP>,
 }
 
+impl<T, A: FixedSizeArray<T>, CHUNK: FixedSizeArray<T>, STUMP: FixedSizeArray<T>> Debug
+    for ArrayChunks<T, A, CHUNK, STUMP>
+where
+    T: Debug,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("ArrayChunks")
+            .field("iter", &self.iter)
+            .field("has_stump", &self.has_stump)
+            .field("_chunk_pd", &self._chunk_pd)
+            .field("_stump_pd", &self._stump_pd)
+            .finish()
+    }
+}
+
+impl<T, A: FixedSizeArray<T>, CHUNK: FixedSizeArray<T>, STUMP: FixedSizeArray<T>> PartialEq
+    for ArrayChunks<T, A, CHUNK, STUMP>
+where
+    T: PartialEq,
+{
+    fn eq(&self, other: &Self) -> bool {
+        self.iter == other.iter && self.has_stump == other.has_stump
+    }
+}
+
+impl<T, A: FixedSizeArray<T>, CHUNK: FixedSizeArray<T>, STUMP: FixedSizeArray<T>> Eq
+    for ArrayChunks<T, A, CHUNK, STUMP>
+where
+    T: Eq,
+{
+}
+
+impl<T, A: FixedSizeArray<T>, CHUNK: FixedSizeArray<T>, STUMP: FixedSizeArray<T>> Clone
+    for ArrayChunks<T, A, CHUNK, STUMP>
+where
+    T: Clone,
+{
+    fn clone(&self) -> Self {
+        ArrayChunks {
+            iter: self.iter.clone(),
+            has_stump: self.has_stump,
+            _chunk_pd: PhantomData,
+            _stump_pd: PhantomData,
+        }
+    }
+}
+
 impl<T, A: FixedSizeArray<T>, CHUNK: FixedSizeArray<T>, STUMP: FixedSizeArray<T>>
     ArrayChunks<T, A, CHUNK, STUMP>
 {
@@ -1139,4 +1297,343 @@ mod tests {
             Some(ArrayChunk::Chunk([11u64, 12], PhantomData))
         );
     }
+
+    #[test]
+    fn fixed_capacity_deque_like_eq() {
+        use super::FixedCapacityDequeLike;
+        let mut deque1: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque1.push_back(1);
+        deque1.push_back(2);
+        deque1.push_back(3);
+        deque1.push_back(4);
+        deque1.push_back(5);
+        deque1.pop_front();
+        deque1.pop_front();
+
+        let mut deque2: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque2.push_back(1);
+        deque2.push_back(2);
+        deque2.push_back(3);
+        deque2.push_back(4);
+        deque2.push_back(5);
+        deque2.pop_front();
+        deque2.pop_front();
+
+        assert_eq!(deque1, deque2);
+    }
+
+    #[test]
+    fn fixed_capacity_deque_like_eq_empty() {
+        use super::FixedCapacityDequeLike;
+
+        let deque1: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        let deque2: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+
+        assert_eq!(deque1, deque2);
+    }
+
+    #[test]
+    fn fixed_capacity_deque_like_not_eq_different_items() {
+        use super::FixedCapacityDequeLike;
+
+        let mut deque1: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque1.push_back(1);
+        deque1.push_back(2);
+
+        let mut deque2: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque2.push_back(2);
+        deque1.push_back(3);
+
+        assert_ne!(deque1, deque2);
+    }
+
+    #[test]
+    fn fixed_capacity_deque_like_not_eq_different_offsets() {
+        use super::FixedCapacityDequeLike;
+
+        let mut deque1: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque1.push_back(0);
+        deque1.push_back(1);
+        deque1.push_back(2);
+        deque1.pop_back();
+
+        let mut deque2: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque2.push_back(1);
+        deque1.push_back(2);
+
+        assert_ne!(deque1, deque2);
+    }
+
+    #[test]
+    fn fixed_capacity_deque_like_clone_no_offset() {
+        use super::FixedCapacityDequeLike;
+
+        let mut deque: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque.push_back(1);
+        deque.push_back(2);
+        deque.push_back(3);
+
+        let clone = deque.clone();
+
+        assert_eq!(deque, clone);
+    }
+
+    #[test]
+    fn fixed_capacity_deque_like_clone_with_offset() {
+        use super::FixedCapacityDequeLike;
+
+        let mut deque: FixedCapacityDequeLike<u64, [u64; 10]> = FixedCapacityDequeLike::new();
+        deque.push_back(1);
+        deque.push_back(2);
+        deque.push_back(3);
+        deque.push_back(4);
+        deque.push_back(5);
+        deque.pop_back();
+        deque.pop_front();
+
+        let clone = deque.clone();
+
+        assert_eq!(deque, clone);
+    }
+
+    #[test]
+    fn array_into_iterator_eq() {
+        use super::ArrayIntoIterator;
+
+        let a = [1, 2, 3, 4, 5];
+        let b = [1, 2, 3, 4, 5];
+
+        let iter_a = ArrayIntoIterator::new(a);
+        let iter_b = ArrayIntoIterator::new(b);
+
+        assert_eq!(iter_a, iter_b);
+    }
+
+    #[test]
+    fn array_into_iterator_eq_if_items_eq() {
+        use super::ArrayIntoIterator;
+        use core::mem;
+
+        let a = [1, 2, 3, 4, 5, 6];
+        let b = [0, 0, 0, 3, 4, 5];
+
+        let mut iter_a = ArrayIntoIterator::new(a);
+        let mut iter_b = ArrayIntoIterator::new(b);
+
+        mem::drop(iter_a.next());
+        mem::drop(iter_a.next());
+        mem::drop(iter_a.next_back());
+
+        mem::drop(iter_b.next());
+        mem::drop(iter_b.next());
+        mem::drop(iter_b.next());
+
+        assert_eq!(iter_a, iter_b);
+    }
+
+    #[test]
+    fn array_into_iterator_clone() {
+        use super::ArrayIntoIterator;
+        let a = [1, 2, 3, 4, 5];
+        let a_iter = ArrayIntoIterator::new(a);
+        let a_iter_clone = a_iter.clone();
+        assert_eq!(a_iter, a_iter_clone);
+    }
+
+    #[test]
+    fn array_chunk_eq_chunk() {
+        use super::ArrayChunk;
+        use core::marker::PhantomData;
+
+        let chunk1: ArrayChunk<u64, [u64; 4], [u64; 3]> =
+            ArrayChunk::Chunk([1, 2, 3, 4], PhantomData);
+        let chunk2: ArrayChunk<u64, [u64; 4], [u64; 3]> =
+            ArrayChunk::Chunk([1, 2, 3, 4], PhantomData);
+
+        assert_eq!(chunk1, chunk2);
+    }
+
+    #[test]
+    fn array_chunk_eq_stump() {
+        use super::ArrayChunk;
+        use core::marker::PhantomData;
+
+        let chunk1: ArrayChunk<u64, [u64; 4], [u64; 3]> = ArrayChunk::Stump([1, 2, 3], PhantomData);
+        let chunk2: ArrayChunk<u64, [u64; 4], [u64; 3]> = ArrayChunk::Stump([1, 2, 3], PhantomData);
+
+        assert_eq!(chunk1, chunk2);
+    }
+
+    #[test]
+    fn array_chunk_ne_stump_and_chunk() {
+        use super::ArrayChunk;
+        use core::marker::PhantomData;
+
+        let chunk1: ArrayChunk<u64, [u64; 4], [u64; 3]> =
+            ArrayChunk::Chunk([1, 2, 3, 4], PhantomData);
+        let chunk2: ArrayChunk<u64, [u64; 4], [u64; 3]> = ArrayChunk::Stump([1, 2, 3], PhantomData);
+
+        assert_ne!(chunk1, chunk2);
+    }
+
+    #[test]
+    fn array_chunk_ne_different_chunk_items() {
+        use super::ArrayChunk;
+        use core::marker::PhantomData;
+
+        let chunk1: ArrayChunk<u64, [u64; 4], [u64; 3]> =
+            ArrayChunk::Chunk([1, 2, 3, 4], PhantomData);
+        let chunk2: ArrayChunk<u64, [u64; 4], [u64; 3]> =
+            ArrayChunk::Chunk([4, 3, 2, 1], PhantomData);
+
+        assert_ne!(chunk1, chunk2);
+    }
+
+    #[test]
+    fn array_chunk_ne_different_stump_items() {
+        use super::ArrayChunk;
+        use core::marker::PhantomData;
+
+        let chunk1: ArrayChunk<u64, [u64; 4], [u64; 3]> = ArrayChunk::Stump([1, 2, 3], PhantomData);
+        let chunk2: ArrayChunk<u64, [u64; 4], [u64; 3]> = ArrayChunk::Stump([4, 3, 2], PhantomData);
+
+        assert_ne!(chunk1, chunk2);
+    }
+
+    #[test]
+    fn array_chunk_clone_chunk() {
+        use super::ArrayChunk;
+        use core::marker::PhantomData;
+
+        let chunk: ArrayChunk<u64, [u64; 4], [u64; 3]> =
+            ArrayChunk::Chunk([1, 2, 3, 4], PhantomData);
+        let chunk_clone = chunk.clone();
+
+        assert_eq!(chunk, chunk_clone);
+    }
+
+    #[test]
+    fn array_chunk_clone_stump() {
+        use super::ArrayChunk;
+        use core::marker::PhantomData;
+
+        let chunk: ArrayChunk<u64, [u64; 4], [u64; 3]> = ArrayChunk::Stump([1, 2, 3], PhantomData);
+        let chunk_clone = chunk.clone();
+
+        assert_eq!(chunk, chunk_clone);
+    }
+
+    #[test]
+    fn array_chunks_eq_case_1() {
+        use super::ArrayChunks;
+        use core::mem;
+
+        let a = [1, 2, 3, 4, 5, 6, 7, 8];
+        let mut a_chunks = ArrayChunks::<u64, [u64; 8], [u64; 3], [u64; 2]>::new(a);
+
+        let b = [1, 2, 3, 4, 5, 6, 7, 8];
+        let mut b_chunks = ArrayChunks::<u64, [u64; 8], [u64; 3], [u64; 2]>::new(b);
+
+        assert_eq!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_eq!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next_back());
+        mem::drop(b_chunks.next_back());
+
+        assert_eq!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_eq!(a_chunks, b_chunks);
+    }
+
+    #[test]
+    fn array_chunks_eq_case_2() {
+        use super::ArrayChunks;
+        use core::mem;
+
+        let a = [0, 1, 2, 4, 5, 6, 8, 9];
+        let mut a_chunks = ArrayChunks::<u64, [u64; 8], [u64; 3], [u64; 2]>::new(a);
+
+        let b = [1, 2, 3, 4, 5, 6, 7, 8];
+        let mut b_chunks = ArrayChunks::<u64, [u64; 8], [u64; 3], [u64; 2]>::new(b);
+
+        assert_ne!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_ne!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next_back());
+        mem::drop(b_chunks.next_back());
+
+        assert_eq!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_eq!(a_chunks, b_chunks);
+    }
+
+    #[test]
+    fn array_chunks_eq_case_3() {
+        use super::ArrayChunks;
+        use core::mem;
+
+        let a = [0, 1, 2, 4, 5, 6, 8, 9];
+        let mut a_chunks = ArrayChunks::<u64, [u64; 8], [u64; 3], [u64; 2]>::new(a);
+
+        let b = [10, 11, 12, 13, 14, 15, 16, 17];
+        let mut b_chunks = ArrayChunks::<u64, [u64; 8], [u64; 3], [u64; 2]>::new(b);
+
+        assert_ne!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_ne!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next_back());
+        mem::drop(b_chunks.next_back());
+
+        assert_ne!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_eq!(a_chunks, b_chunks);
+    }
+
+    #[test]
+    fn array_chunks_clone() {
+        use super::ArrayChunks;
+        use core::mem;
+
+        let a = [1, 2, 3, 4, 5, 6, 7, 8];
+        let mut a_chunks = ArrayChunks::<u64, [u64; 8], [u64; 3], [u64; 2]>::new(a);
+        let mut b_chunks = a_chunks.clone();
+
+        assert_eq!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_eq!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next_back());
+        mem::drop(b_chunks.next_back());
+
+        assert_eq!(a_chunks, b_chunks);
+
+        mem::drop(a_chunks.next());
+        mem::drop(b_chunks.next());
+
+        assert_eq!(a_chunks, b_chunks);
+    }
 }
